# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-16 09:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ['number']},
        ),
        migrations.AlterModelOptions(
            name='part',
            options={'ordering': ['number'], 'verbose_name': '\u5c0f\u8282'},
        ),
        migrations.AlterModelOptions(
            name='question',
            options={'ordering': ['number']},
        ),
        migrations.AlterModelOptions(
            name='section',
            options={'ordering': ['number'], 'verbose_name': 'course\u8bfe\u4ef6'},
        ),
        migrations.AlterModelOptions(
            name='video',
            options={'ordering': ['number']},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-21 07:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0007_auto_20190418_1409'),
    ]

    operations = [
        migrations.AddField(
            model_name='usersection',
            name='started_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-10 09:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0016_auto_20190510_1652'),
    ]

    operations = [
        migrations.AlterField(
            model_name='section',
            name='group_name',
            field=models.CharField(blank=True, help_text='\u5982\u8bfe\u4ef6\u524d\u4e0d\u9700\u663e\u793a\uff0c\u5219\u4e0d\u7528\u586b\u5199', max_length=180, null=True, verbose_name='\u7ec4\u540d'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-18 06:09
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bee_django_course_simple', '0006_auto_20190418_1318'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserCourse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('status', models.IntegerField(default=0, verbose_name='\u72b6\u6001')),
                ('passed_at', models.DateTimeField(blank=True, null=True)),
                ('course', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Course', verbose_name='\u8bfe\u7a0b')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='simple_course_user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-created_at'],
                'db_table': 'bee_django_course_simple_user_course',
            },
        ),
        migrations.CreateModel(
            name='UserPart',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('passed_at', models.DateTimeField(blank=True, null=True)),
                ('status', models.IntegerField(default=0)),
                ('part', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Part')),
            ],
            options={
                'ordering': ['part__number'],
                'db_table': 'bee_django_course_simple_user_part',
            },
        ),
        migrations.CreateModel(
            name='UserQuestion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('passed_at', models.DateTimeField(auto_now_add=True)),
                ('answer_option_id', models.IntegerField(null=True)),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Question')),
                ('user_part', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.UserPart')),
            ],
            options={
                'ordering': ['question__number'],
                'db_table': 'bee_django_course_simple_user_question',
            },
        ),
        migrations.CreateModel(
            name='UserSection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('passed_at', models.DateTimeField(blank=True, null=True)),
                ('status', models.IntegerField(default=0)),
                ('section', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Section')),
                ('user_course', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.UserCourse')),
            ],
            options={
                'ordering': ['section__number'],
                'db_table': 'bee_django_course_simple_user_section',
            },
        ),
        migrations.CreateModel(
            name='UserVideo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('passed_at', models.DateTimeField(auto_now_add=True)),
                ('user_part', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.UserPart')),
                ('video', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Video')),
            ],
            options={
                'ordering': ['video__number'],
                'db_table': 'bee_django_course_simple_user_video',
            },
        ),
        migrations.AddField(
            model_name='userpart',
            name='user_section',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.UserSection'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.core.urlresolvers import reverse
from django.utils import timezone
from django.conf import settings
from django.dispatch import receiver
from django.db.models.signals import post_save
from smart_selects.db_fields import ChainedForeignKey


# Create your models here.
# 课程
class Course(models.Model):
    title = models.CharField(max_length=180, verbose_name='课程标题')  # 名字
    subtitle = models.CharField(max_length=180, null=True, verbose_name='课程副标题', blank=True)  # 副标题
    level = models.IntegerField(default=0, verbose_name='课程的level', blank=True)  # 课程的level
    is_del = models.IntegerField(default=0)
    is_auto_open = models.BooleanField(default=False, verbose_name='是否自动开启')
    created_at = models.DateTimeField(default=timezone.now)

    class Meta:
        db_table = 'bee_django_course_simple_course'
        app_label = 'bee_django_course_simple'
        ordering = ["-id"]
        permissions = [
            ('can_manage_course', '可以进入课程管理页'),
            ('view_all_courses', '可以查看所有课程'),
        ]

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('bee_django_course_simple:course_detail', kwargs={'pk': self.pk})


SECTION_TYPE_CHOICES = ((1, '普通课件'), (2, '预备课件'))


# 课件
class Section(models.Model):
    course = models.ForeignKey(Course, verbose_name='属于的课程')
    group_name = models.CharField(max_length=180, verbose_name='组名', null=True, blank=True, help_text='如课件前不需显示，则不用填写')
    # pre_title =models.CharField(max_length=180, verbose_name='标题前缀',null=True,blank=True)
    title = models.CharField(max_length=180, verbose_name='课件标题')  # 名字
    extra_title = models.CharField(max_length=180, verbose_name='额外标题', null=True, blank=True)  # 名字
    number = models.IntegerField(verbose_name='排序', default=1)  # 排序
    type = models.IntegerField(choices=SECTION_TYPE_CHOICES, default=1, verbose_name='课件类型')  #
    auto_pass = models.BooleanField(default=False, verbose_name='自动通过')
    info = models.TextField(null=True, blank=True, verbose_name='介绍')
    created_at = models.DateTimeField(default=timezone.now)

    class Meta:
        db_table = 'bee_django_course_simple_section'
        app_label = 'bee_django_course_simple'
        ordering = ["number"]
        permissions = [
            ('view_all_sections', '可以查看所有课件'),
        ]

    def __unicode__(self):
        t = self.title
        if self.extra_title:
            t += self.extra_title
        return t

    def get_absolute_url(self):
        return reverse('bee_django_course_simple:section_detail', kwargs={'pk': self.pk})

    def get_type(self):
        if self.type == 1:
            return '普通课件'
        elif self.type == 2:
            return '预备课件'
        return ''

    def get_auto_pass(self):
        if self.auto_pass:
            return '是'
        else:
            return '否'


# 小节
class Part(models.Model):
    section = models.ForeignKey(Section, verbose_name='属于的课件')
    image = models.ImageField(null=True, blank=True, verbose_name='配图', upload_to='bee_django_course_simple/part_image')
    # pre_title =models.CharField(max_length=180, verbose_name='标题前缀',null=True,blank=True)
    title = models.CharField(verbose_name='标题', max_length=180)
    extra_title = models.CharField(verbose_name='额外标题', max_length=180, null=True, blank=True)
    number = models.IntegerField(verbose_name='排序', default=1)  # 排序
    type = models.IntegerField(verbose_name='类型', default=1)  # 1-视频 2-单选问题
    has_answer = models.BooleanField(default=False, verbose_name='是否有正确答案')

    class Meta:
        db_table = 'bee_django_course_simple_part'
        app_label = 'bee_django_course_simple'
        ordering = ["number"]
        verbose_name = '小节'

    def __unicode__(self):
        t = self.title
        if self.extra_title:
            t += self.extra_title
        return t

    def get_type(self):
        if self.type == 1:
            return '视频'
        elif self.type == 2:
            if self.has_answer:
                return '选择题'
            elif not self.has_answer:
                return '无答案选择题'
        return ''

    def is_video_type(self):
        if self.type == 1:
            return True
        return False

    def is_question_type(self):
        if self.type == 2:
            return True
        return False


# 问题
class Question(models.Model):
    course = models.ForeignKey(Course, null=True, verbose_name='课程', blank=True)
    section = ChainedForeignKey(
        Section,
        chained_field="course",
        chained_model_field="course",
        show_all=False,
        auto_choose=True,
        sort=True,
        null=True,
        blank=True,
        verbose_name='课件')
    part = ChainedForeignKey(
        Part,
        chained_field="section",
        chained_model_field="section",
        show_all=False,
        auto_choose=True,
        sort=True,
        null=True,
        blank=True,
        verbose_name='小节')
    # part = models.ForeignKey(Part, verbose_name='小节')  # 属于哪个小节
    # city = ChainedForeignKey(
    #     City,
    #     chained_field="province",
    #     chained_model_field="province",
    #     show_all=False,
    #     auto_choose=True,
    #     sort=True,
    #     null=True,
    #     blank=True,
    #     verbose_name='市')
    number = models.IntegerField(verbose_name='排序', default=1)  # 排序
    title = models.CharField(max_length=180, verbose_name='问题', null=True)  # 问题
    tip_wrong = models.TextField(null=True, verbose_name="错误时提示词")
    tip_correct = models.TextField(null=True, verbose_name="正确时提示词")

    class Meta:
        db_table = 'bee_django_course_simple_question'
        app_label = "bee_django_course_simple"
        ordering = ['number']
        permissions = (
            ('view_question', '可以查看问题列表'),
        )

    # 获取答题的正确答案
    def get_correct_options(self):
        return Option.objects.filter(question=self, is_correct=True)

    # 获取【无】正确答案的问题
    @classmethod
    def get_has_correct_questions(cls, has_correct=False):
        if has_correct:
            return cls.objects.filter(part__has_answer=True)
        else:
            return cls.objects.exclude(part__has_answer=True)


# 选项
class Option(models.Model):
    question = models.ForeignKey(Question, verbose_name='问题')
    title = models.CharField(max_length=180, verbose_name='选项')
    number = models.IntegerField(default=0, verbose_name='顺序')
    is_correct = models.BooleanField(default=False, verbose_name='是否正确答案')

    class Meta:
        db_table = 'bee_django_course_simple_option'
        app_label = "bee_django_course_simple"
        ordering = ['number']


class QuestionPrize(models.Model):
    correct_count = models.IntegerField(verbose_name='答对问题的数量', unique=True)
    coin = models.IntegerField(verbose_name='奖励M币')

    class Meta:
        db_table = 'bee_django_course_simple_question_prize'
        app_label = 'bee_django_course_simple'
        ordering = ['correct_count']

    def __unicode__(self):
        return 'correct_count：' + self.correct_count.__str__() + "，coin：" + self.coin.__str__()

    @classmethod
    def get_coin(cls, correct_count):
        try:
            c = cls.objects.get(correct_count=correct_count)
            return c.coin
        except:
            return 0


# 视频
class Video(models.Model):
    part = models.ForeignKey(Part, verbose_name='小节')  # 属于哪个小节
    number = models.IntegerField(verbose_name='排序', default=1)  # 排序
    file_name = models.CharField(max_length=180, verbose_name='视频文件名')
    content = models.TextField(verbose_name='内容')

    class Meta:
        db_table = 'bee_django_course_simple_video'
        app_label = "bee_django_course_simple"
        ordering = ['number']


# 富文本上传的图片
class UserImage(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, related_name='simple_user_image')
    image = models.ImageField(verbose_name='图片', upload_to='bee_django_course_simple/video_image/%Y/%m/%d')
    upload_at = models.DateTimeField(verbose_name='上传时间', auto_now_add=True)
    model_name = models.CharField(max_length=180, verbose_name='model名', null=True)
    model_id = models.IntegerField(null=True)

    class Meta:
        db_table = 'bee_django_course_simple_user_image'
        app_label = 'bee_django_course_simple'


# ======学生相关======
class UserCourse(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='simple_course_user')
    course = models.ForeignKey(Course, verbose_name='课程')
    created_at = models.DateTimeField(auto_now_add=True)
    status = models.IntegerField(default=0, verbose_name='状态')  # 0 学习中，1 已完成
    passed_at = models.DateTimeField(null=True, blank=True)

    class Meta:
        db_table = 'bee_django_course_simple_user_course'
        app_label = 'bee_django_course_simple'
        ordering = ['-created_at']
        permissions = [
            ('assign_user_course', '能给学生分配课程'),
        ]

    # 给学生添加自动添加的课程
    @classmethod
    def auto_add(cls, user):
        for course in Course.objects.filter(is_auto_open=True):
            user_course = UserCourse()
            user_course.user = user
            user_course.course = course
            user_course.save()

    # 为学生添加course下的所有section
    def add_user_sections(self):
        section_list = self.course.section_set.all()
        for section in section_list:
            us = UserSection()
            us.user_course = self
            us.section = section
            us.status = 0
            us.save()
        return


class UserSection(models.Model):
    user_course = models.ForeignKey(UserCourse)
    section = models.ForeignKey(Section)
    created_at = models.DateTimeField(auto_now_add=True)
    started_at = models.DateTimeField(null=True, blank=True)
    passed_at = models.DateTimeField(null=True, blank=True)
    status = models.IntegerField(default=0)  # 0 未开始， 1 学习中，2 通过

    class Meta:
        db_table = 'bee_django_course_simple_user_section'
        app_label = 'bee_django_course_simple'
        ordering = ['section__number']
        permissions = [
            ('pass_ucs', '能通过学生课件'),
            ('close_ucs', '能关闭学生课件'),
            ('open_ucs', '能开启学生课件'),
        ]

    # 为学生添加section下的所有part
    def add_user_parts(self):
        part_list = self.section.part_set.all()
        for part in part_list:
            up = UserPart()
            up.user_section = self
            up.part = part
            up.status = 0
            up.save()
        return

    def next_user_section(self, status=None):
        number = self.section.number
        section_list = UserSection.objects.filter(user_course=self.user_course, section__number__gt=number)
        if not status == None:
            section_list = section_list.filter(status=status)
        if section_list.exists():
            return section_list.first()
        return None

    # 判断该section是否可以通过
    def can_pass(self):
        if not self.is_open() == True:
            return False
        if self.section.auto_pass == True:
            return True
        part_list = self.section.part_set.all()
        for part in part_list:
            user_part = UserPart.objects.filter(user_section=self, part=part, passed_at__isnull=True)
            if user_part.exists():
                return False
        return True

    def _open(self):
        self.started_at = timezone.now()
        self.status = 1
        self.save()
        part_list = self.userpart_set.all()
        if part_list.exists():
            part_list.first()._open()
        return

    def _pass(self):
        self.passed_at = timezone.now()
        self.status = 2
        self.save()
        # user section下的所有user part也通过
        for user_part in self.userpart_set.all():
            user_part._pass()
        # 检查下一个section开启
        next_section = self.next_user_section(status=0)
        if next_section:
            next_section._open()
            if next_section.can_pass():
                next_section._pass()
        return

    def _close(self):
        self.status = 0
        self.save()
        # user section下的所有user part也关闭
        for user_part in self.userpart_set.all():
            user_part._close()
        return

    def is_close(self):
        return self.status == 0

    def is_open(self):
        return self.status == 1

    def is_pass(self):
        return self.status == 2


class UserPart(models.Model):
    user_section = models.ForeignKey(UserSection)
    part = models.ForeignKey(Part)
    created_at = models.DateTimeField(auto_now_add=True)
    passed_at = models.DateTimeField(null=True, blank=True)
    status = models.IntegerField(default=0)  # 0 未开始， 1 学习中，2 通过
    prize_coin = models.IntegerField(default=0, verbose_name='奖励的m币')
    question_correct_count = models.IntegerField(default=0, verbose_name='答对了几道题')

    class Meta:
        db_table = 'bee_django_course_simple_user_part'
        app_label = 'bee_django_course_simple'
        ordering = ['part__number']

    def next_user_part(self, status=None):
        number = self.part.number
        part_list = UserPart.objects.filter(user_section=self.user_section, part__number__gt=number)
        if not status == None:
            part_list = part_list.filter(status=status)
        if part_list.exists():
            return part_list.first()
        return None

    def _open(self):
        self.status = 1
        self.save()
        return

    def _pass(self):
        self.status = 2
        self.passed_at = timezone.now()
        self.save()
        next_part = self.next_user_part(status=0)
        if next_part:
            next_part._open()
        return

    def _close(self):
        self.status = 0
        self.save()
        return

    def is_close(self):
        return self.status == 0

    def is_open(self):
        return self.status == 1

    def is_pass(self):
        return self.status == 2

    # 判断该小节是否可以通过
    def can_pass(self):
        if not self.is_open():
            return False
        video_list = self.part.video_set.all()
        for video in video_list:
            user_video = UserVideo.objects.filter(user_part=self, video=video)
            if not user_video.exists():
                return False
        question_list = self.part.question_set.all()
        for question in question_list:
            user_question = UserQuestion.objects.filter(user_part=self, question=question)
            if not user_question.exists():
                return False
        return True



@receiver(post_save, sender=UserCourse)
def add_user_sections(sender, **kwargs):
    user_course = kwargs['instance']
    if kwargs['created']:
        user_course.add_user_sections()
        user_section_list = user_course.usersection_set.all()
        if user_section_list.exists():
            user_section = user_section_list.first()
            user_section._open()
            if user_section.section.auto_pass == True:
                user_section._pass()
    return


@receiver(post_save, sender=UserSection)
def add_user_parts(sender, **kwargs):
    user_section = kwargs['instance']
    if kwargs['created']:
        user_section.add_user_parts()
        user_part_list = user_section.userpart_set.all()
        if user_part_list.exists():
            user_part = user_part_list.first()
            user_part._open()
    return


class UserVideo(models.Model):
    user_part = models.ForeignKey(UserPart)
    video = models.ForeignKey(Video)
    passed_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        db_table = 'bee_django_course_simple_user_video'
        app_label = 'bee_django_course_simple'
        ordering = ['video__number']


class UserQuestion(models.Model):
    user_part = models.ForeignKey(UserPart)
    question = models.ForeignKey(Question)
    passed_at = models.DateTimeField(auto_now_add=True)
    answer_option = models.ForeignKey(Option, null=True)

    class Meta:
        db_table = 'bee_django_course_simple_user_question'
        app_label = 'bee_django_course_simple'
        ordering = ['question__number']



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-20 12:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0020_auto_20190515_1427'),
    ]

    operations = [
        migrations.AddField(
            model_name='section',
            name='auto_pass',
            field=models.BooleanField(default=False, verbose_name='\u81ea\u52a8\u901a\u8fc7'),
        ),
        migrations.AddField(
            model_name='section',
            name='type',
            field=models.IntegerField(choices=[(1, '\u666e\u901a\u8bfe\u4ef6'), (2, '\u9884\u5907\u8bfe\u4ef6')], default=1, verbose_name='\u8bfe\u4ef6\u7c7b\u578b'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-10 08:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0015_section_pre_title'),
    ]

    operations = [
        migrations.AddField(
            model_name='part',
            name='pre_title',
            field=models.CharField(max_length=180, null=True, verbose_name='\u6807\u9898\u524d\u7f00'),
        ),
        migrations.AddField(
            model_name='section',
            name='group_name',
            field=models.CharField(help_text='\u5982\u8bfe\u4ef6\u524d\u4e0d\u9700\u663e\u793a\uff0c\u5219\u4e0d\u7528\u586b\u5199', max_length=180, null=True, verbose_name='\u7ec4\u540d'),
        ),
        migrations.AlterField(
            model_name='course',
            name='title',
            field=models.CharField(max_length=180, verbose_name='\u8bfe\u7a0b\u6807\u9898'),
        ),
        migrations.AlterField(
            model_name='section',
            name='pre_title',
            field=models.CharField(max_length=180, null=True, verbose_name='\u6807\u9898\u524d\u7f00'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title',
            field=models.CharField(max_length=180, verbose_name='\u8bfe\u4ef6\u6807\u9898'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-10 05:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0014_course_is_auto_open'),
    ]

    operations = [
        migrations.AddField(
            model_name='section',
            name='pre_title',
            field=models.CharField(max_length=180, null=True, verbose_name='\u540d\u5b57\u524d\u7f00'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import render

import json, os, random

from django.shortcuts import get_object_or_404, reverse, redirect, render, HttpResponse
from django.contrib.auth import get_user_model
from django.http import JsonResponse, HttpResponseRedirect
from django.views.generic import ListView, DetailView, TemplateView, RedirectView

from django.conf import settings
from dss.Serializer import serializer
from .models import Course, Section, Part, Video, Question, Option, QuestionPrize, UserCourse, UserSection, \
    UserPart, UserVideo, UserQuestion

from .signals import add_prize_coin

User = get_user_model()


class UserPartUpdate(TemplateView):
    def post(self, request, *args, **kwargs):
        user_part_id = self.kwargs["user_part_id"]
        question_data = self.request.POST.get("question_data")
        video_id = self.request.POST.get("video_id")

        next_user_part_id = None
        # next_user_section_id = None
        # next_user_section_url = None
        next_user_part_btn = False  # 下一节按钮
        user_part = UserPart.objects.get(id=user_part_id)
        # question_count = 0  # 小节中题目数量
        question_correct_count = 0  # 小节中答对题目数量
        if not user_part.is_open():
            return JsonResponse(data={
                'error': 1,
                'message': '失败，原因：该小节已通过',
            })

        # 保存记录,更新part内question或video状态
        if question_data:
            question_data = json.loads(question_data)
            for question in question_data:
                # 记录过则不记录
                u_q = UserQuestion.objects.filter(user_part=user_part, question_id=question['question_id'])
                if u_q.exists():
                    continue
                user_question = UserQuestion()
                user_question.user_part = user_part
                user_question.question_id = question["question_id"]
                user_question.answer_option_id = question["option_id"]
                user_question.save()

                # # 该小节问题的数量
                # question_count = Question.objects.filter(part=user_part.part).count()
                # 检查答案
                corrent_option = Option.objects.filter(question__id=question["question_id"], id=question["option_id"],
                                                       is_correct=True)
                if corrent_option:
                    question_correct_count += 1
            # 奖励m币
            prize_coin = QuestionPrize.get_coin(question_correct_count)
            user_part.prize_coin = prize_coin
            user_part.question_correct_count = question_correct_count
            user_part.save()
            # 发送信号
            if prize_coin > 0:
                reason = "答对题目奖励"
                add_prize_coin.send(sender=UserPart, user=user_part.user_section.user_course.user, coin=prize_coin, reason=reason)

        if video_id:
            video = Video.objects.get(id=video_id)
            # 记录过则不记录
            u_v = UserVideo.objects.filter(user_part=user_part, video=video)
            if not u_v.exists():
                video = Video.objects.get(id=video_id)
                user_video = UserVideo()
                user_video.user_part = user_part
                user_video.video = video
                user_video.save()

        # 判断当前user part是否可以通过，更新user part 状态
        user_part_can_pass = user_part.can_pass()
        if user_part_can_pass:
            user_part._pass()

        # 查找下一个user part
        next_user_part = user_part.next_user_part(status=1)
        if next_user_part:
            next_user_part_id = next_user_part.id
            next_user_part_btn = True

        # 判断当前user section是否可以通过
        user_section_can_pass = user_part.user_section.can_pass()
        if user_section_can_pass:
            user_part.user_section._pass()

        # 查找下一个user section，检查自动开启
        # next_user_section = user_part.user_section.next_user_section(status=0)
        # if next_user_section and user_section_can_pass:
        #     next_user_section._open()
        #     next_user_section_id = next_user_section.id
        #     next_user_section_url = reverse('bee_django_course_simple:custom_user_section_detail',
        #                                     kwargs={'pk': next_user_section_id})

        return JsonResponse(data={
            'error': 0,
            'message': '成功',
            'user_part_status': user_part.status,
            'next_user_part_id': next_user_part_id,
            'next_user_part_btn': next_user_part_btn,
            # 'next_user_section_id': next_user_section_id,
            # 'next_user_section_url': next_user_section_url,
            # 'user_part_can_pass': user_part_can_pass,
            "user_section_can_pass": user_section_can_pass,
            # "question_count": question_count,
            # "question_correct_count": question_correct_count,
        })


# VueJS 拉取数据用
class VueUserSection(TemplateView):
    def get(self, request, *args, **kwargs):
        user_section_id = self.kwargs['user_section_id']
        user_section = get_object_or_404(UserSection, pk=user_section_id)
        user_parts = user_section.userpart_set.all()

        for user_part in user_parts:
            user_part.part_type = user_part.part.type
            user_part.part_title = user_part.part.title
            user_part.extra_title = user_part.part.extra_title

            if user_part.part_type == 1:
                videos = []
                for v in user_part.part.video_set.all():
                    videos.append({
                        'id': v.id,
                        'content': v.content,
                        'url': "http://" + settings.QINIU_DOMAIN + "/" + v.file_name
                    })
                user_part.videos = videos
            elif user_part.part_type == 2:
                questions = []
                for q in user_part.part.question_set.all():
                    options = []
                    for o in q.option_set.all():
                        options.append({
                            'id': o.id,
                            'title': o.title
                        })
                    questions.append({
                        'id': q.id,
                        'title': q.title,
                        'options': options
                    })
                user_part.questions = questions

        data = serializer(user_parts, output_type='json', datetime_format='string')

        return JsonResponse(data={
            'user_parts': data,
            'status': user_section.status,
        })



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-23 07:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0013_auto_20190421_1848'),
    ]

    operations = [
        migrations.AddField(
            model_name='course',
            name='is_auto_open',
            field=models.BooleanField(default=False, verbose_name='\u662f\u5426\u81ea\u52a8\u5f00\u542f'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

# Register your models here.
from django.contrib import admin
from .models import QuestionPrize

class QuestionPrizeAdmin(admin.ModelAdmin):
    # 需要显示的字段信息
    list_display = ('id', 'correct_count', 'coin')

    # 设置哪些字段可以点击进入编辑界面，默认是第一个字段
    list_display_links = ('id',)

# 注册时，在第二个参数写上 admin model
admin.site.register(QuestionPrize, QuestionPrizeAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-18 05:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0005_userimage'),
    ]

    operations = [
        migrations.AddField(
            model_name='userimage',
            name='model_id',
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name='userimage',
            name='model_name',
            field=models.CharField(max_length=180, null=True, verbose_name='model\u540d'),
        ),
        migrations.AlterField(
            model_name='userimage',
            name='image',
            field=models.ImageField(upload_to='bee_django_course_simple/video_image/%Y/%m/%d', verbose_name='\u56fe\u7247'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-16 08:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Course',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=180, verbose_name='\u8bfe\u7a0b\u540d\u5b57')),
                ('subtitle', models.CharField(blank=True, max_length=180, null=True, verbose_name='\u8bfe\u7a0b\u526f\u6807\u9898')),
                ('level', models.IntegerField(blank=True, default=0, verbose_name='\u8bfe\u7a0b\u7684level')),
                ('is_del', models.IntegerField(default=0)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'ordering': ['-id'],
                'db_table': 'bee_django_course_simple_course',
                'verbose_name': 'course\u8bfe\u7a0b',
            },
        ),
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=180, verbose_name='\u9009\u9879')),
                ('number', models.IntegerField(default=0, verbose_name='\u987a\u5e8f')),
                ('is_correct', models.BooleanField(default=False, verbose_name='\u662f\u5426\u6b63\u786e\u7b54\u6848')),
            ],
            options={
                'ordering': ['pk'],
                'db_table': 'bee_django_course_simple_option',
            },
        ),
        migrations.CreateModel(
            name='Part',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=180, verbose_name='\u6807\u9898')),
                ('number', models.IntegerField(default=1, verbose_name='\u6392\u5e8f')),
                ('type', models.IntegerField(default=1, verbose_name='\u7c7b\u578b')),
                ('has_answer', models.BooleanField(default=False, verbose_name='\u662f\u5426\u6709\u6b63\u786e\u7b54\u6848')),
            ],
            options={
                'ordering': ['-id'],
                'db_table': 'bee_django_course_simple_part',
                'verbose_name': '\u5c0f\u8282',
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('number', models.IntegerField(default=1, verbose_name='\u6392\u5e8f')),
                ('title', models.CharField(max_length=180, null=True, verbose_name='\u95ee\u9898')),
                ('tip_wrong', models.TextField(null=True, verbose_name='\u9519\u8bef\u65f6\u63d0\u793a\u8bcd')),
                ('tip_correct', models.TextField(null=True, verbose_name='\u6b63\u786e\u65f6\u63d0\u793a\u8bcd')),
                ('part', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Part', verbose_name='\u5c0f\u8282')),
            ],
            options={
                'ordering': ['pk'],
                'db_table': 'bee_django_course_simple_question',
            },
        ),
        migrations.CreateModel(
            name='Section',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=180, verbose_name='\u8bfe\u4ef6\u540d\u5b57')),
                ('number', models.IntegerField(default=1, verbose_name='\u6392\u5e8f')),
                ('info', models.TextField(blank=True, null=True, verbose_name='\u4ecb\u7ecd')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('course', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Course', verbose_name='\u5c5e\u4e8e\u7684\u8bfe\u7a0b')),
            ],
            options={
                'ordering': ['-id'],
                'db_table': 'bee_django_course_simple_section',
                'verbose_name': 'course\u8bfe\u4ef6',
            },
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('number', models.IntegerField(default=1, verbose_name='\u6392\u5e8f')),
                ('content', models.TextField(verbose_name='\u5185\u5bb9')),
                ('part', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Part', verbose_name='\u5c0f\u8282')),
                ('url', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Question', verbose_name='\u89c6\u9891\u5730\u5740')),
            ],
            options={
                'ordering': ['pk'],
                'db_table': 'bee_django_course_simple_video',
            },
        ),
        migrations.AddField(
            model_name='part',
            name='section',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Section', verbose_name='\u5c5e\u4e8e\u7684\u8bfe\u4ef6'),
        ),
        migrations.AddField(
            model_name='option',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Question', verbose_name='\u95ee\u9898'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-15 06:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0019_auto_20190513_1315'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='userquestion',
            name='answer_option_id',
        ),
        migrations.AddField(
            model_name='userquestion',
            name='answer_option',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Option'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-21 09:52
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0008_usersection_started_at'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='course',
            options={'ordering': ['-id'], 'permissions': [('can_manage_course', '\u53ef\u4ee5\u8fdb\u5165\u8bfe\u7a0b\u7ba1\u7406\u9875'), ('view_all_courses', '\u53ef\u4ee5\u67e5\u770b\u6240\u6709\u8bfe\u7a0b')]},
        ),
        migrations.AlterModelOptions(
            name='question',
            options={'ordering': ['number'], 'permissions': (('view_question', '\u53ef\u4ee5\u67e5\u770b\u95ee\u9898\u5217\u8868'),)},
        ),
        migrations.AlterModelOptions(
            name='section',
            options={'ordering': ['number'], 'permissions': [('view_all_sections', '\u53ef\u4ee5\u67e5\u770b\u6240\u6709\u8bfe\u4ef6')]},
        ),
        migrations.AlterModelOptions(
            name='usercourse',
            options={'ordering': ['-created_at'], 'permissions': [('assign_user_course', '\u80fd\u7ed9\u5b66\u751f\u5206\u914d\u8bfe\u7a0b')]},
        ),
        migrations.AlterModelOptions(
            name='usersection',
            options={'ordering': ['section__number'], 'permissions': [('pass_ucs', '\u80fd\u901a\u8fc7\u5b66\u751f\u8bfe\u4ef6'), ('close_ucs', '\u80fd\u5173\u95ed\u5b66\u751f\u8bfe\u4ef6'), ('open_ucs', '\u80fd\u5f00\u542f\u5b66\u751f\u8bfe\u4ef6')]},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-13 05:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0017_auto_20190510_1710'),
    ]

    operations = [
        migrations.AddField(
            model_name='part',
            name='extra_title',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u989d\u5916\u6807\u9898'),
        ),
        migrations.AddField(
            model_name='section',
            name='extra_title',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u989d\u5916\u6807\u9898'),
        ),
        migrations.AlterField(
            model_name='part',
            name='pre_title',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u6807\u9898\u524d\u7f00'),
        ),
        migrations.AlterField(
            model_name='section',
            name='pre_title',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u6807\u9898\u524d\u7f00'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-21 10:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import smart_selects.db_fields


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0011_question_course'),
    ]

    operations = [
        migrations.AlterField(
            model_name='question',
            name='course',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Course', verbose_name='\u8bfe\u7a0b'),
        ),
        migrations.AlterField(
            model_name='question',
            name='part',
            field=smart_selects.db_fields.ChainedForeignKey(auto_choose=True, blank=True, chained_field='section', chained_model_field='section', null=True, on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Part', verbose_name='\u5c0f\u8282'),
        ),
        migrations.AlterField(
            model_name='question',
            name='section',
            field=smart_selects.db_fields.ChainedForeignKey(auto_choose=True, blank=True, chained_field='course', chained_model_field='course', null=True, on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Section', verbose_name='\u8bfe\u4ef6'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-24 11:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0023_auto_20190524_1705'),
    ]

    operations = [
        migrations.AddField(
            model_name='userpart',
            name='question_correct_count',
            field=models.IntegerField(default=0, verbose_name='\u7b54\u5bf9\u4e86\u51e0\u9053\u9898'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-24 09:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0022_part_image'),
    ]

    operations = [
        migrations.CreateModel(
            name='QuestionPrize',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('correct_count', models.IntegerField(unique=True, verbose_name='\u7b54\u5bf9\u95ee\u9898\u7684\u6570\u91cf')),
                ('coin', models.IntegerField(verbose_name='\u5956\u52b1M\u5e01')),
            ],
            options={
                'ordering': ['correct_count'],
                'db_table': 'bee_django_course_simple_question_prize',
            },
        ),
        migrations.AddField(
            model_name='userpart',
            name='prize_coin',
            field=models.IntegerField(default=0, verbose_name='\u5956\u52b1\u7684m\u5e01'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'zhangyue'

from django import template
from bee_django_course_simple.models import UserQuestion,Question


register = template.Library()

# 求两个值的差的绝对值
@register.filter
def get_difference_abs(a, b):
    return abs(a - b)

# 用户答题
@register.simple_tag
def get_user_answer_option(user, question_id):
    question = Question.objects.get(id=question_id)
    try:
        user_question = UserQuestion.objects.get(question=question,user_part__user_section__user_course__user=user)
        answer_option=user_question.answer_option
        return answer_option.id
    except:
        return None


#!/usr/bin/env python
# -*- coding:utf-8 -*-

__author__ = 'bee'
import json,pytz
from django.conf import settings
from django.apps import apps
from django.contrib.auth.models import User
from django.http import HttpResponse
from datetime import datetime
from django.conf import settings

LOCAL_TIMEZONE = pytz.timezone('Asia/Shanghai')




class JSONResponse(HttpResponse):
    def __init__(self, obj):
        if isinstance(obj, dict):
            _json_str = json.dumps(obj)
        else:
            _json_str = obj
        super(JSONResponse, self).__init__(_json_str, content_type="application/json;charset=utf-8")

# ====dt====
# 获取本地当前时间
def get_now(tz=LOCAL_TIMEZONE):
    return datetime.now(tz)



def get_user_model():
    if settings.COIN_USER_TABLE in ["", None]:
        user_model = User
    else:
        app_name = settings.COIN_USER_TABLE.split(".")[0]
        model_name = settings.COIN_USER_TABLE.split(".")[1]
        app = apps.get_app_config(app_name)
        user_model = app.get_model(model_name)
    return user_model


# 获取登录用户
def get_login_user(request):
    if settings.COIN_USER_TABLE in ["", None]:
        return request.user

    token = request.COOKIES.get('cookie_token', '')
    # 没有登录
    if not token:
        return None

    try:
        user_table = get_user_model()
        user = user_table.objects.get(token=token)
        return user
    except:
        return None


# 获取自定义user的自定义name
def get_user_name(user):
    try:
        return getattr(user, settings.COIN_USER_NAME_FIELD)
    except:
        return None

def get_default_name():
    return settings.COIN_DEFAULT_NAME


# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-20 12:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0021_auto_20190520_2014'),
    ]

    operations = [
        migrations.AddField(
            model_name='part',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='bee_django_course_simple/part_image', verbose_name='\u914d\u56fe'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-21 10:26
from __future__ import unicode_literals

from django.db import migrations
import django.db.models.deletion
import smart_selects.db_fields


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0009_auto_20190421_1752'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='section',
            field=smart_selects.db_fields.ChainedForeignKey(auto_choose=True, blank=True, chained_field='part', chained_model_field='part', null=True, on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Section', verbose_name='\u8bfe\u4ef6'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from django.conf.urls import include, url
from . import views,apis

app_name = 'bee_django_course_simple'
urlpatterns = [
    url(r'^test$', views.test, name='test'),
    url(r'^$', views.CourseList.as_view(), name='index'),
    # =======课程========
    url(r'^course/list$', views.CourseList.as_view(), name='course_list'),
    url(r'^course/detail/(?P<pk>[0-9]+)$', views.CourseDetail.as_view(), name='course_detail'),
    url(r'^course/add$', views.CourseCreate.as_view(), name='course_add'),
    url(r'^course/update/(?P<pk>[0-9]+)$', views.CourseUpdate.as_view(), name='course_update'),
    url(r'^course/delete/(?P<pk>[0-9]+)$', views.CourseDelete.as_view(), name='course_delete'),

    # =======课件========
    url(r'^section/list$', views.SectionList.as_view(), name='section_list'),
    url(r'^section/detail/(?P<pk>\d+)$', views.SectionDetail.as_view(), name='section_detail'),
    url(r'^section/add/(?P<course_id>[0-9]+)$', views.SectionCreate.as_view(), name='section_add'),
    url(r'^section/update/(?P<pk>[0-9]+)$', views.SectionUpdate.as_view(), name='section_update'),
    url(r'^section/delete/(?P<pk>[0-9]+)$', views.SectionDelete.as_view(), name='section_delete'),

    # =======小节========
    url(r'^part/list$', views.PartList.as_view(), name='part_list'),
    url(r'^part/detail/(?P<pk>[0-9]+)$', views.PartDetail.as_view(), name='part_detail'),
    url(r'^part/add/(?P<section_id>[0-9]+)$', views.PartCreate.as_view(), name='part_add'),
    url(r'^part/update/(?P<pk>[0-9]+)$', views.PartUpdate.as_view(), name='part_update'),
    url(r'^part/delete/(?P<pk>[0-9]+)$', views.PartDelete.as_view(), name='part_delete'),

    # =======视频========
    url(r'^uptoken$', views.uptoken, name='uptoken'),
    url(r'^parts/(?P<part_id>\d+)/add_video$', views.add_video_to_part, name='add_video_to_part'),
    url(r'^video/(?P<video_id>\d+)/edit$', views.edit_video_content, name='edit_video_content'),
    url(r'^video/(?P<video_id>\d+)/delete$', views.delete_video, name='delete_video'),
    url(r'^video/(?P<video_id>\d+)$', views.video_detail, name='video_detail'),
    url(r'^upload_image$', views.upload_image, name='upload_image'),

    # ========== 选择题 ==============
    url(r'^question/list$', views.QuestionList.as_view(), name='question_list'),
    url(r'^question/detail/(?P<pk>[0-9]+)$', views.QuestionDetail.as_view(), name='question_detail'),
    url(r'^question/record/detail/(?P<pk>[0-9]+)$', views.QuestionRecordDetail.as_view(), name='question_record_detail'),
    url(r'^question/add/(?P<part_id>[0-9]+)$', views.QuestionCreate.as_view(), name='question_add'),
    url(r'^question/update/(?P<pk>[0-9]+)$', views.QuestionUpdate.as_view(), name='question_update'),
    url(r'^question/delete/(?P<pk>[0-9]+)$', views.QuestionDelete.as_view(), name='question_delete'),
    url(r'^option/add/(?P<question_id>[0-9]+)$', views.OptionCreate.as_view(), name='option_add'),
    url(r'^option/update/(?P<pk>[0-9]+)$', views.OptionUpdate.as_view(), name='option_update'),
    url(r'^option/delete/(?P<pk>[0-9]+)$', views.OptionDelete.as_view(), name='option_delete'),


    # ========== 用户课件 ==============
    url(r'^user/course/add/(?P<user_id>[0-9]+)$', views.UserCourseCreate.as_view(), name='user_course_add'),
    url(r'^user/course/delete/(?P<pk>[0-9]+)$', views.UserCourseDelete.as_view(), name='user_course_delete'),
    url(r'^user/section/list/(?P<user_id>[0-9]+)/(?P<user_course_id>[0-9]+)$', views.UserSectionList.as_view(), name='user_section_list'),
    url(r'^user/section/detail/(?P<pk>[0-9]+)$', views.UserSectionDetail.as_view(), name='user_section_detail'),
    url(r'^user/section/update/(?P<user_section_id>[0-9]+)/(?P<type>(.)+)$', views.UserSectionUpdate.as_view(), name='user_section_update'),
    url(r'^user/part/update/(?P<user_part_id>[0-9]+)$', apis.UserPartUpdate.as_view(), name='user_part_update'),

    # ========== 用户前台 ==============
    url(r'^custom_user/section/redirect/(?P<page_type>(.)+)/(?P<user_id>[0-9]+)/(?P<section_type>[0-9]+)/$', views.CustomUserSectionRedirect.as_view(), name='custom_user_section_redirect'),
    url(r'^custom_user/section/list/(?P<section_type>[0-9]+)/(?P<user_id>[0-9]+)/(?P<user_course_id>[0-9]+)$', views.CustomUserSectionList.as_view(), name='custom_user_section_list'),
    url(r'^custom_user/section/detail/(?P<section_type>[0-9]+)/(?P<pk>[0-9]+)$', views.CustomUserSectionDetail.as_view(), name='custom_user_section_detail'),
    url(r'^custom_user/section/empty$', views.CustomUserSectionEmpty.as_view(template_name="bee_django_course_simple/section/custom_user_section_empty.html"), name='custom_user_section_empty'),
    url(r'^custom_user/part/detail/video/(?P<pk>[0-9]+)$', views.CustomUserPartDetailVideo.as_view(), name='custom_user_part_detail_video'),
    url(r'^custom_user/part/detail/question/(?P<pk>[0-9]+)$', views.CustomUserPartDetailQuestion.as_view(), name='custom_user_part_detail_question'),


    # ============ VUE JS ===============
    # url(r'^vue_user_section/(?P<user_section_id>\d+)$', apis.VueUserSection.as_view(), name='vue_user_section'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-21 10:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0012_auto_20190421_1846'),
    ]

    operations = [
        migrations.AlterField(
            model_name='question',
            name='course',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='bee_django_course_simple.Course', verbose_name='\u8bfe\u7a0b'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-13 05:15
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0018_auto_20190513_1312'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='part',
            name='pre_title',
        ),
        migrations.RemoveField(
            model_name='section',
            name='pre_title',
        ),
    ]



# -*- coding:utf-8 -*-
__author__ = 'bee'

from django import forms
from .models import Course, Section, Part, Video, Question, Option, UserImage, UserCourse


# ===== course contract======
class CourseForm(forms.ModelForm):
    class Meta:
        model = Course
        fields = ['title', "subtitle", "is_auto_open"]


# ===== section contract======
class SectionForm(forms.ModelForm):
    class Meta:
        model = Section
        fields = ['group_name', 'title', 'extra_title', 'type', 'number', 'auto_pass']


# ===== part contract======
class PartForm(forms.ModelForm):
    type = forms.ChoiceField(choices=((1, '视频'), (2, "选择题")), label='小节类型')

    class Meta:
        model = Part
        fields = ['title', "extra_title", 'number', 'type', 'image', 'has_answer']


class PartUpdateForm(forms.ModelForm):
    class Meta:
        model = Part
        fields = ['title', 'extra_title', 'number', 'image', 'has_answer']


# ===== part contract======
class VideoForm(forms.ModelForm):
    class Meta:
        model = Video
        fields = ['number', 'content']


class UploadImageForm(forms.ModelForm):
    class Meta:
        model = UserImage
        fields = ['image']


# ===== question option contract======
class QuestionForm(forms.ModelForm):
    class Meta:
        model = Question
        fields = ['number', 'title']


class QuestionSearchForm(forms.ModelForm):
    title = forms.CharField(label='问题', required=False)

    class Meta:
        model = Question
        fields = ['course', 'section', 'title']


class OptionForm(forms.ModelForm):
    class Meta:
        model = Option
        fields = ['number', 'title', "is_correct"]


# =====用户相关========
class UserCourseForm(forms.ModelForm):
    class Meta:
        model = UserCourse
        fields = ['course']



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-17 09:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_course_simple', '0003_auto_20190416_1737'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='video',
            name='url',
        ),
        migrations.AddField(
            model_name='video',
            name='file_name',
            field=models.CharField(default='', max_length=180, verbose_name='\u89c6\u9891\u6587\u4ef6\u540d'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import render

import json, os, random

from django.core.urlresolvers import reverse_lazy
from django.utils.decorators import method_decorator
from django.shortcuts import get_object_or_404, reverse, redirect, render, HttpResponse
from django.views.generic import ListView, DetailView, TemplateView, RedirectView
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.http import JsonResponse, HttpResponseRedirect
from django.db import transaction
from django.core.exceptions import PermissionDenied
from django.db.models import Q, Sum, Count

from qiniu import Auth
from qiniu import BucketManager

from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required, permission_required
from dss.Serializer import serializer
from django.db.models.signals import pre_delete
from django.dispatch import receiver

from .models import Course, Section, Part, Video, Question, Option, UserCourse, UserSection, \
    UserPart, UserVideo, UserQuestion
from .forms import CourseForm, SectionForm, PartForm, PartUpdateForm, VideoForm, QuestionForm, \
    QuestionSearchForm, OptionForm, UploadImageForm, UserCourseForm

User = get_user_model()


# Create your views here.
def test(request):
    q = Question.objects.all()
    for i in q:
        i.course = i.part.section.course
        i.section = i.part.section
        i.save()
    return


# =======course=======
class CourseList(ListView):
    template_name = 'bee_django_course_simple/course/course_list.html'
    context_object_name = 'course_list'
    paginate_by = 20
    queryset = Course.objects.all()

    def get(self, request, *args, **kwargs):
        if not request.user.has_perm('bee_django_course_simple.view_all_courses'):
            self.queryset = Course.objects.none()
        return super(CourseList, self).get(request, *args, **kwargs)


class CourseDetail(DetailView):
    model = Course
    template_name = 'bee_django_course_simple/course/course_detail.html'
    context_object_name = 'course'


@method_decorator(permission_required('bee_django_course_simple.add_course'), name='dispatch')
class CourseCreate(CreateView):
    model = Course
    form_class = CourseForm
    template_name = 'bee_django_course_simple/course/course_form.html'


class CourseUpdate(UpdateView):
    model = Course
    form_class = CourseForm
    template_name = 'bee_django_course_simple/course/course_form.html'


class CourseDelete(DeleteView):
    model = Course
    success_url = reverse_lazy('bee_django_course_simple:course_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


class SectionList(ListView):
    template_name = 'bee_django_course_simple/section/section_list.html'
    context_object_name = 'section_list'
    paginate_by = 20
    queryset = Section.objects.all()

    def get(self, request, *args, **kwargs):
        if request.user.has_perm("bee_django_course.view_all_sections"):
            return super(SectionList, self).get(request, *args, **kwargs)
        else:
            raise PermissionDenied

    def get_queryset(self):
        section_name = self.request.GET.get('section_name')
        if section_name:
            return self.queryset.filter(name__contains=section_name)
        else:
            return self.queryset


class SectionDetail(DetailView):
    model = Section
    template_name = 'bee_django_course_simple/section/section_detail.html'
    context_object_name = 'section'


class SectionCreate(CreateView):
    model = Section
    form_class = SectionForm
    template_name = 'bee_django_course_simple/section/section_form.html'

    def get_context_data(self, **kwargs):
        context = super(SectionCreate, self).get_context_data(**kwargs)
        context['course'] = Course.objects.get(id=self.kwargs["course_id"])
        context['type'] = 'add'
        return context

    def form_valid(self, form):
        form.instance.course_id = self.kwargs["course_id"]
        return super(SectionCreate, self).form_valid(form)


class SectionUpdate(UpdateView):
    model = Section
    form_class = SectionForm
    template_name = 'bee_django_course_simple/section/section_form.html'

    def get_context_data(self, **kwargs):
        context = super(SectionUpdate, self).get_context_data(**kwargs)
        section = Section.objects.get(id=self.kwargs["pk"])
        context['course'] = section.course
        context['type'] = 'update'
        return context


class SectionDelete(DeleteView):
    model = Section
    success_url = reverse_lazy('bee_django_course_simple:course_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


class PartList(ListView):
    template_name = 'bee_django_course_simple/part/part_list.html'
    context_object_name = 'part_list'
    paginate_by = 20
    queryset = Part.objects.all()

    def get(self, request, *args, **kwargs):
        if request.user.has_perm("bee_django_course_simple.view_all_parts"):
            return super(PartList, self).get(request, *args, **kwargs)
        else:
            raise PermissionDenied

            # def get_queryset(self):
            #     section_name = self.request.GET.get('section_name')
            #     if section_name:
            #         return self.queryset.filter(name__contains=section_name)
            #     else:
            #         return self.queryset


class PartDetail(DetailView):
    model = Part
    template_name = 'bee_django_course_simple/part/part_detail.html'
    context_object_name = 'part'


class PartCreate(CreateView):
    model = Part
    form_class = PartForm
    template_name = 'bee_django_course_simple/part/part_form.html'
    part = None

    def get_context_data(self, **kwargs):
        context = super(PartCreate, self).get_context_data(**kwargs)
        context['section'] = Section.objects.get(id=self.kwargs["section_id"])
        context['type'] = 'add'
        return context

    def form_valid(self, form):
        form.instance.section_id = self.kwargs["section_id"]
        self.part = form.instance
        return super(PartCreate, self).form_valid(form)

    def get_success_url(self):
        if 'next' in self.request.POST:
            if self.part.type == 1:
                return reverse_lazy("bee_django_course_simple:video_add", kwargs={'part_id': self.part.id})
            elif self.part.type == 2:
                return reverse_lazy("bee_django_course_simple:question_add", kwargs={'part_id': self.part.id})
        else:
            return reverse_lazy("bee_django_course_simple:section_detail", kwargs={'pk': self.part.section.id})


class PartUpdate(UpdateView):
    model = Part
    form_class = PartUpdateForm
    template_name = 'bee_django_course_simple/part/part_form.html'
    part = None

    def get_context_data(self, **kwargs):
        context = super(PartUpdate, self).get_context_data(**kwargs)
        part = Part.objects.get(id=self.kwargs["pk"])
        context['section'] = part.section
        context['type'] = 'update'
        return context

    def form_valid(self, form):
        self.part = form.instance
        return super(PartUpdate, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy('bee_django_course_simple:section_detail', kwargs={'pk': self.part.section.id})


class PartDelete(DeleteView):
    model = Part
    success_url = reverse_lazy('bee_django_course_simple:course_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


# ==========Video==========
# 上传视频到七牛，需要先获取的token
def get_qiniu_token(key):
    access_key = settings.QINIU_AK
    secret_key = settings.QINIU_SK
    # 构建鉴权对象
    q = Auth(access_key, secret_key)
    # 要上传的空间
    bucket_name = settings.QINIU_BUCKET_NAME
    # key 上传后保存的文件名

    # 生成上传 Token，可以指定过期时间等
    # 上传策略示例
    # https://developer.qiniu.com/kodo/manual/1206/put-policy
    policy = {
        # 'callbackUrl':'https://requestb.in/1c7q2d31',
        # 'callbackBody':'filename=$(fname)&filesize=$(fsize)'
        # 'persistentOps':'imageView2/1/w/200/h/200'
    }
    # 3600为token过期时间，秒为单位。3600等于一小时
    token = q.upload_token(bucket_name, key=key.encode('utf-8'), expires=3600, policy=policy)
    return token


def uptoken(request):
    key = request.GET.get('key')
    token = get_qiniu_token(key)
    return JsonResponse(data={
        'uptoken': token,
        'domain': settings.QINIU_DOMAIN,
    })


def add_video_to_part(request, part_id):
    if request.method == "POST":
        file_name = request.POST.get('file_name')
        try:
            part = get_object_or_404(Part, pk=part_id)
            video_count = part.video_set.all().count() + 1
            part.video_set.create(file_name=file_name, number=video_count)

            return JsonResponse(data={
                'rc': 0,
                'message': '创建成功'
            })
        except Part.DoesNotExist:
            return JsonResponse(data={
                'rc': -1,
                'message': '未找到对应小节'
            })


# 编辑小节视频的文字
def edit_video_content(request, video_id):
    video = get_object_or_404(Video, pk=video_id)
    if request.method == "POST":
        form = VideoForm(data=request.POST, instance=video)
        if form.is_valid():
            form.save()
            return redirect(reverse('bee_django_course_simple:part_detail', kwargs={'pk': video.part.id}))
        else:
            pass
    else:
        form = VideoForm(instance=video)

    return render(request, 'bee_django_course_simple/video/video_form.html', context={
        'form': form,
    })


# 视频内容富文本图片上传
@csrf_exempt
def upload_image(request):
    max_size = settings.COURSE_UPLOAD_MAXSIZE
    if request.method == "POST":
        file = request.FILES.get(settings.COURSE_SIMPLE_ATTACH_FILENAME)
        if file.size > max_size:
            return HttpResponse("error|图片大小超过5M!")

        # 保存图片。用户上传的图片，与用户的对应关系也保存到数据库中
        form = UploadImageForm(request.POST, request.FILES)
        if form.is_valid():
            user_image = form.save(commit=False)
            if request.user.is_authenticated:
                user_image.user = request.user
            user_image.save()
            return HttpResponse(user_image.image.url)
        else:
            print form.errors
            return HttpResponse("error|文件存储错误")
    else:
        return HttpResponse("error|请求错误")


def video_detail(request, video_id):
    video = get_object_or_404(Video, pk=video_id)

    return render(request, 'bee_django_course_simple/video/video_detail.html', context={
        'video': video,
        'url': u'http://' + settings.QINIU_DOMAIN + u'/' + video.file_name
    })


def delete_video(request, video_id):
    if request.method == "POST":
        video = get_object_or_404(Video, pk=video_id)

        if request.user.has_perm('bee_django_course_simple.change_section'):
            video.delete()
            return JsonResponse(data={
                'rc': 0,
                'message': '删除成功'
            })
        else:
            return JsonResponse(data={
                'rc': -1,
                'message': '权限不足'
            })


@receiver(pre_delete, sender=Video, dispatch_uid='video_delete_singal')
def delete_qiniu_video(sender, instance, using, **kwargs):
    key = instance.file_name

    access_key = settings.QINIU_AK
    secret_key = settings.QINIU_SK
    # 构建鉴权对象
    q = Auth(access_key, secret_key)
    # 初始化BucketManager
    bucket = BucketManager(q)

    # 要上传的空间
    bucket_name = settings.QINIU_BUCKET_NAME

    # 删除bucket_name 中的文件 key
    ret, info = bucket.delete(bucket_name, key.encode('utf-8'))
    # print(info)


# ========== 小节为问题==============
class QuestionCreate(CreateView):
    model = Question
    form_class = QuestionForm
    template_name = 'bee_django_course_simple/question/question_form.html'
    question = None

    def get_context_data(self, **kwargs):
        context = super(QuestionCreate, self).get_context_data(**kwargs)
        context['part'] = Part.objects.get(id=self.kwargs["part_id"])
        return context

    def form_valid(self, form):
        form.instance.part_id = self.kwargs["part_id"]
        part = Part.objects.get(id=self.kwargs["part_id"])
        form.instance.section_id = part.section.id
        form.instance.course_id = part.section.course.id
        self.question = form.instance
        return super(QuestionCreate, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy("bee_django_course_simple:option_add", kwargs={"question_id": self.question.id})


# =======选择题==========
class QuestionList(ListView):
    template_name = 'bee_django_course_simple/question/question_list.html'
    context_object_name = 'question_list'
    paginate_by = 20
    http_method_names = [u'get', u"post"]
    queryset = Question.objects.all()

    def search(self):
        course_id = self.request.GET.get("course")
        section_id = self.request.GET.get("section")
        title = self.request.GET.get("title")

        # 检查权限
        if not self.request.user.has_perm("bee_django_course_simple.view_question"):
            self.queryset = Question.objects.none()
            return self.queryset

        if not course_id in ["", 0, None, "0"]:
            self.queryset = self.queryset.filter(part__section__course__id=course_id)
        if not section_id in ["", 0, None, "0"]:
            self.queryset = self.queryset.filter(part__section__id=section_id)
        if not title in ["", 0, None]:
            self.queryset = self.queryset.filter(title__icontains=title)

        return self.queryset

    def get_context_data(self, **kwargs):
        context = super(QuestionList, self).get_context_data(**kwargs)
        course_id = self.request.GET.get("course")
        section_id = self.request.GET.get("section")
        title = self.request.GET.get("title")

        context['search_form'] = QuestionSearchForm(
            {"course": course_id, "section": section_id, "title": title})
        return context

    def get(self, request, *args, **kwargs):
        self.queryset = self.search()
        return super(QuestionList, self).get(request, *args, **kwargs)


class QuestionDetail(DetailView):
    model = Question
    template_name = 'bee_django_course_simple/question/question_detail.html'
    context_object_name = 'question'


class QuestionRecordDetail(DetailView):
    model = Question
    template_name = 'bee_django_course_simple/question/question_reocrd_detail.html'
    context_object_name = 'question'

    def get_context_data(self, **kwargs):
        context = super(QuestionRecordDetail, self).get_context_data(**kwargs)
        question_id = self.kwargs["pk"]
        question = Question.objects.get(id=question_id)
        option_list = Option.objects.filter(question=question)
        res_queryset = UserQuestion.objects.filter(question=question).values('answer_option_id').annotate(
            answer_count=Count('answer_option')).order_by("-answer_count")

        res_list = []
        user_count = 0
        for option in option_list:
            d = {}
            found = False
            for i in res_queryset:
                answer_option_id = i["answer_option_id"]
                if not answer_option_id:
                    found = False
                elif option.id == answer_option_id:
                    d["key"] = option.title
                    d["value"] = i["answer_count"]
                    user_count += i["answer_count"]
                    found = True
                    res_list.append(d)
            if found == False:
                d["key"] = option.title
                d["value"] = 0
                res_list.append(d)

        # for res_dict in res_queryset:
        #     # option_id = res_dict["answer"]
        #     count = res_dict["answer_count"]

        context["user_count"] = user_count
        context["res_list"] = res_list
        # context["res_queryset"]=res_queryset
        return context


class QuestionUpdate(UpdateView):
    model = Question
    form_class = QuestionForm
    template_name = 'bee_django_course_simple/question/question_form.html'

    def get_context_data(self, **kwargs):
        context = super(QuestionUpdate, self).get_context_data(**kwargs)
        question = Question.objects.get(id=self.kwargs["pk"])
        context['part'] = question.part
        return context

    def get_success_url(self):
        question = Question.objects.get(id=self.kwargs["pk"])
        return reverse_lazy("bee_django_course_simple:part_detail", kwargs={"pk": question.part.id})


class QuestionDelete(DeleteView):
    model = Question
    success_url = reverse_lazy('bee_django_course_simple:course_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


class OptionCreate(CreateView):
    model = Option
    form_class = OptionForm
    template_name = 'bee_django_course_simple/question/option_form.html'
    question = None

    def get_context_data(self, **kwargs):
        context = super(OptionCreate, self).get_context_data(**kwargs)
        question = Question.objects.get(id=self.kwargs["question_id"])
        context['question'] = question
        context['type'] = 'add'
        return context

    def form_valid(self, form):
        form.instance.question_id = self.kwargs["question_id"]
        self.question = form.instance.question
        return super(OptionCreate, self).form_valid(form)

    def get_success_url(self):
        if 'next' in self.request.POST:
            return reverse_lazy("bee_django_course_simple:option_add", kwargs=self.kwargs)
        else:
            return reverse_lazy("bee_django_course_simple:section_detail", kwargs={"pk": self.question.part.section.id})


class OptionUpdate(UpdateView):
    model = Option
    form_class = OptionForm
    template_name = 'bee_django_course_simple/question/option_form.html'

    def get_context_data(self, **kwargs):
        context = super(OptionUpdate, self).get_context_data(**kwargs)
        option = Option.objects.get(id=self.kwargs["pk"])
        context['question'] = option.question
        context['type'] = 'update'
        return context

    def get_success_url(self):
        option = Option.objects.get(id=self.kwargs["pk"])
        return reverse_lazy("bee_django_course_simple:part_detail", kwargs={"pk": option.question.part.id})


class OptionDelete(DeleteView):
    model = Option
    success_url = reverse_lazy('bee_django_course_simple:course_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


# ========用户课件列表=========

# 给用户添加新课程
class UserCourseCreate(CreateView):
    model = UserCourse
    form_class = UserCourseForm
    template_name = 'bee_django_course_simple/course/user_course_form.html'
    user_course = None

    # def get_context_data(self, **kwargs):
    #     context = super(OptionCreate, self).get_context_data(**kwargs)
    #     question = Question.objects.get(id=self.kwargs["question_id"])
    #     context['question'] = question
    #     context['type'] = 'add'
    #     return context
    #
    def form_valid(self, form):
        form.instance.user_id = self.kwargs["user_id"]
        self.user_course = form.instance
        return super(UserCourseCreate, self).form_valid(form)

    #
    def get_success_url(self):
        return reverse_lazy("bee_django_course_simple:user_section_list",
                            kwargs={"user_id": self.kwargs["user_id"], "user_course_id": self.user_course.id})


class UserCourseDelete(DeleteView):
    model = UserCourse
    success_url = reverse_lazy('bee_django_course_simple:course_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


# 查看指定用户user的课件列表
class UserSectionList(ListView):
    template_name = "bee_django_course_simple/section/user_section_list.html"
    model = UserSection
    queryset = None
    paginate_by = 30
    context_object_name = 'user_section_list'
    user_course = None
    user_course_list = []

    def get_section_type_list(self):
        return [1, 2]  # 展示全部课件

    def get_queryset(self):
        user_id = self.kwargs["user_id"]
        user_course_id = self.kwargs["user_course_id"]
        self.user_course_list = UserCourse.objects.filter(user__id=user_id).order_by('status', '-created_at')
        if not user_course_id in [None, 0, "0", u"0"]:
            self.user_course = UserCourse.objects.get(id=user_course_id)
        else:
            if self.user_course_list.exists():
                self.user_course = self.user_course_list.first()
        if self.user_course:
            return UserSection.objects.filter(user_course=self.user_course).filter(
                section__type__in=self.get_section_type_list())
        else:
            return UserSection.objects.none()

    def get_context_data(self, **kwargs):
        context = super(UserSectionList, self).get_context_data(**kwargs)
        user = User.objects.get(id=self.kwargs["user_id"])
        context["user"] = user
        context["user_course"] = self.user_course
        context["user_course_list"] = self.user_course_list
        return context


class UserSectionDetail(DetailView):
    model = UserSection
    template_name = 'bee_django_course_simple/section/user_section_detail.html'
    context_object_name = 'user_section'


# ======学生前台页面========

class CustomUserSectionRedirect(RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        user_id = self.kwargs['user_id']
        section_type = self.kwargs['section_type']
        page_type = self.kwargs['page_type']

        user_course_list = UserCourse.objects.filter(user__id=user_id).order_by("status")
        if user_course_list.exists():
            user_course = user_course_list.first()
            #  只找正在学的和已通过的。
            user_section_list = user_course.usersection_set.filter(status__gte=1).order_by("status", "section__number")
            if user_course_list.exists():
                user_section = user_section_list.first()
                if page_type == 'detail':
                    self.url = reverse('bee_django_course_simple:custom_user_section_detail',
                                       kwargs={"pk": user_section.id, "section_type": section_type})
                elif page_type == 'list':
                    self.url = reverse('bee_django_course_simple:custom_user_section_list',
                                       kwargs={"section_type": section_type, "user_course_id": user_course.id,
                                               "user_id": user_id})

            else:
                self.url = reverse('bee_django_course_simple:custom_user_section_empty')

                # print('====', self.url)
        else:
            self.url = reverse('bee_django_course_simple:custom_user_section_empty')

        return super(CustomUserSectionRedirect, self).get_redirect_url(*args, **kwargs)


# 普通课件列表，或预备课列表
class CustomUserSectionList(UserSectionList):
    def get_section_type_list(self):
        return [self.kwargs["section_type"]]

    def get_template_names(self):
        section_type = self.kwargs["section_type"]
        if section_type in [1, "1"]:
            return "bee_django_course_simple/section/custom_user_section_list1.html"
        elif section_type in [2, "2"]:
            return "bee_django_course_simple/section/custom_user_section_list2.html"


class CustomUserSectionDetail(UserSectionDetail):
    def get_template_names(self):
        section_type = self.kwargs["section_type"]
        if section_type in [1, "1"]:
            return 'bee_django_course_simple/section/custom_user_section_detail1.html'
        elif section_type in [2, "2"]:
            return "bee_django_course_simple/section/custom_user_section_detail2.html"

    def get_context_data(self, **kwargs):
        context = super(CustomUserSectionDetail, self).get_context_data(**kwargs)
        user_section = UserSection.objects.get(id=self.kwargs["pk"])
        context["next_user_section"] = user_section.next_user_section()
        context["section_type"] = self.kwargs["section_type"]
        return context


class CustomUserSectionEmpty(TemplateView):
    template_name = 'bee_django_course_simple/section/custom_user_section_empty.html'


class CustomUserPartDetail(DetailView):
    model = UserPart
    context_object_name = 'user_part'

    def get_context_data(self, **kwargs):
        context = super(CustomUserPartDetail, self).get_context_data(**kwargs)
        user_part = UserPart.objects.get(id=self.kwargs["pk"])
        next_user_part = user_part.next_user_part()
        if user_part.is_pass() and next_user_part:
            context["next_user_part_btn"] = True
        else:
            context["next_user_part_btn"] = False
        context["qiniu_domain"] = settings.QINIU_DOMAIN
        context["next_user_part"] = user_part.next_user_part()
        return context


class CustomUserPartDetailVideo(CustomUserPartDetail):
    template_name = 'bee_django_course_simple/part/custom_user_part_detail.html'


class CustomUserPartDetailQuestion(CustomUserPartDetail):
    template_name = 'bee_django_course_simple/part/custom_user_part_detail.html'

    def get_context_data(self, **kwargs):
        context = super(CustomUserPartDetailQuestion, self).get_context_data(**kwargs)
        user_part = UserPart.objects.get(id=self.kwargs["pk"])
        context["user_answer_list"] = []
        return context


class UserSectionUpdate(TemplateView):
    def post(self, request, *args, **kwargs):
        user_section_id = self.kwargs["user_section_id"]
        type = self.kwargs["type"]
        user_section = UserSection.objects.get(id=user_section_id)
        msg = ''
        if type == 'open':
            user_section._open()
            msg = '开启'
        elif type == 'pass':
            user_section._pass()

            msg = '通过'
        elif type == 'close':
            user_section._close()
            msg = '关闭'
        return JsonResponse(data={
            'rc': 0,
            'message': msg + '成功'
        })
